function pred = fun_calc_pred(conc_singles, g_singles, conc_comb)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% That script accept dose response of N single drugs and return the
% predicted response of combinations at a desired sets of concentrations
% by each of Bliss, Loewe and Fisher models of additivity.
% The inputs are:
% 1) Cell array with N cells containing the concentrations of each of the 
% N single drugs (single drug data).
% 2) Cell array with N cells containing the fitness (0-1) of microbes grow 
% under in each of the conc_singles concentrations (single drug data).
% 3) M by N matrix with the concentrations of all N drugs in all M
% combinations (drug combinations data).
% The outpu is:
% M by 4 matrix where for each of the M combinations the predicted fitness
% is calculated by:
% 1) Bliss model in its multiplicative form (adequate for yield measurements)
% 2) Bliss model in its aditive form (adequate for growth rate measurements)
% 3) Loewe model of additivity.
% 4) Fisher inspired dosage orthogonality model.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

hill_ft = fittype('1./(1+(x/d0).^h)', 'independent', 'x');
% preposition the single drugs struct
n_drugs = numel(conc_singles);
hill_fit = cell(n_drugs,1);
ICs = zeros(99,n_drugs);

for i = 1:n_drugs
    [hill_fit{i}, ICs(:,i)] = calc_hill(conc_singles{i}, g_singles{i}, hill_ft);
end

pred = zeros(size(conc_comb,1),4);

for i = 1:size(conc_comb,1)
    % calc Bliss predictions
    conc = conc_comb(i,:);
    g_all = zeros(n_drugs,1);
    for j = 1:n_drugs
        g_all(j) = hill_fit{j}(conc(j));
    end
    pred(i,1) = prod(g_all);
    pred(i,2) = sum(g_all)-n_drugs+1;
    
    %% calc Loewe and fisher    
    ratio_a_IC = repmat(conc,99,1)./ICs;
    pred(i,3) = interp1(1-sum(ratio_a_IC,2), 0.99:-0.01:0.01, 0);
    pred(i,4) = interp1(1-sum(ratio_a_IC.^2,2), 0.99:-0.01:0.01, 0);
end
end



function [hill_fit, ICs] = calc_hill(conc, g, hill_ft)
[~, d0] = min(abs(g-0.5));
fit_opt = fitoptions( 'Method','NonlinearLeastSquares',...
    'Upper', [inf 15], ...
    'Lower', [0 0.5], ...
    'StartPoint', [d0  2], ...
    'Robust', 'LAR', ...
    'Display', 'off');
hill_fit = fit(conc, g, hill_ft, fit_opt);
ICs = hill_fit.d0*(1./(0.99:-0.01:0.01)-1).^(1/hill_fit.h);
end